'use strict';

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

var _lodashObjectHas = require('lodash/object/has');

var _lodashObjectHas2 = _interopRequireDefault(_lodashObjectHas);

'use strict';

function CustomDrag(opt) {

    opt = opt || {};

    var md, isOver, isDrag, waitingMoveEvent, waitingMoveRaf;

    if (opt.deTarget) {

        opt.deTarget.addEventListener('mousedown', onDown);
        opt.deTarget.addEventListener('mouseover', onEnter);
        opt.deTarget.addEventListener('mouseleave', onLeave);
    }

    this.emitDown = function (e) {

        onDown(e);
    };

    function onDown(e) {

        if (e.button !== 0) {

            return;
        }

        isDrag = true;

        var custom = call('onDown', [e]);

        if (custom === false) {
            //prevent dragging

            window.addEventListener('mouseup', onUpWithoutDrag);

            return;
        }

        // e.stopPropagation() //ex. prevent to drag the parent if that's draggable too
        // - prevent to set the down state on the dragged item

        e.preventDefault(); //ex. prevent selecting text, focusing input

        md = custom || {};

        md.mx = e.clientX;
        md.my = e.clientY;
        md.dx = 0;
        md.dy = 0;

        window.addEventListener('mousemove', onMove);
        window.addEventListener('mouseup', onUp);
        window.addEventListener('mouseleave', onUp);
    }

    function onMove(e) {

        md.__moved = true;

        waitingMoveEvent = e;

        if (!waitingMoveRaf) {

            waitingMoveRaf = window.requestAnimationFrame(rafOnMove);
        }
    }

    function rafOnMove() {

        window.cancelAnimationFrame(waitingMoveRaf);

        var wme = waitingMoveEvent;
        waitingMoveRaf = undefined;
        waitingMoveEvent = undefined;

        var mx = wme.clientX,
            my = wme.clientY;

        md.dx = mx - md.mx;
        md.dy = my - md.my;

        call(['onMove', 'onDrag'], [md, mx, my]);
    }

    function onUp(e) {

        window.removeEventListener('mousemove', onMove);
        window.removeEventListener('mouseup', onUp);
        window.removeEventListener('mouseleave', onUp);

        if (waitingMoveEvent) {
            rafOnMove();
        }

        isDrag = false;
        if (!isOver) {
            onLeave();
        }

        call('onUp', [md, e.clientX, e.clientY, e]);

        if (!md.__moved) {

            call('onClick', [e]);
        }
    }

    function onUpWithoutDrag(e) {

        window.removeEventListener('mouseup', onUpWithoutDrag);

        call('onClick', [e]);
    }

    function onEnter() {

        isOver = true;

        if (!isDrag) {
            call('onEnter');
        }
    }

    function onLeave() {

        isOver = false;

        if (!isDrag) {
            call('onLeave');
        }
    }

    function call(name, args) {

        if (name instanceof Array) {

            name.forEach(function (name) {

                call(name, args);
            });

            return;
        }

        if (name in opt) {

            return opt[name].apply(opt.thisArg, args);
        }
    }
}

module.exports = CustomDrag;
var p = CustomDrag.prototype;

p.destroy = function () {
    //TODO
};
//# sourceMappingURL=CustomDrag.js.map